﻿function updateGoGate(e)
	local ai = e.ai

	-- ゲートへ行く準備
	if ai.phase == 0 then
		local vs = VisitorSystem.Instance

		-- -- ゲートまで歩く
		local x = vs.gateXi_ * World.world.map.stepX
		local z = vs.gateYi_ * World.world.map.stepZ
		ai.AddCommandMoveRoute(x - 5 - World.world.map.stepX * 7, z)
		ai.AddCommandPhase(2)

		-- ai.phase = 1
	-- ゲートへ移動中
	elseif ai.phase == 1 then
	
	-- ゲートへ着いた
	elseif ai.phase == 2 then
		-- 街を訪ねる
		AISystem.ChangeState(e, HumanComponent.STATE_VISIT_TOWN)
	end
end

function updateVisitTown(e)
	local ai = e.ai;

	-- subject  = 棚

	-- subject2 = 商品
	-- subject3 = レジ

	-- 近くの看板を探す
	if ai.phase == 0 then
		-- 半径以内の看板・防壁玄関・玄関をランダムに取得する
		local signboardData = MapChipData.FirstDataCategoryOf(MapChipData.CATEGORY_SIGNBOARD);
		local signboard = MapSystem.GetMapChipRandamInRadius(e.cell.xi, e.cell.yi, signboardData.facilityData.validRange, e.visitor.signboardRoute, false, MapChipData.CATEGORY_SIGNBOARD, MapChipData.CATEGORY_BARRIER_ENTRANCE, MapChipData.CATEGORY_ENTRANCE);

		-- それらが近くになければ、あきらめて帰る
		if signboard == nil then
		
--						ai.AddCommandFukidashi(FukidashiComponent.ID_IKISAKIGANAI);
			ai.AddCommandWait(20);
			ai.AddCommandState(HumanComponent.STATE_RETURN_EXIT);
		-- 見つかったら、そこまで移動する
		else
			e.visitor.signboardRoute.Add(signboard);

			ai.AddCommandMoveRoute(signboard.cell.xi * World.world.map.stepX, signboard.cell.yi * World.world.map.stepZ);
			ai.AddCommandPhase(2);

			if ai.subject2 ~= nil then
				-- 商品を運ぶアニメへ変更
				AnimationSet.ChangeAnimation(e, AnimationSet.BEHAVIOR_LIFT_PRODUCT);
			else
				-- 歩行アニメへ変更
				AnimationSet.ChangeAnimation(e, AnimationSet.BEHAVIOR_EQUIP_WALK);
			end
		end

		ai.phase = 1;
	
	-- あきらめた/移動中
	elseif ai.phase == 1 then
	
	-- 目標に到着
	elseif ai.phase == 2 then
		local shelf = nil;

		-- 玄関だったら、お店に到着とする
		local chip = MapSystem.GetMapChip(e.cell.xi, e.cell.yi);

		if chip.mapChip.data.type == MapChipData.TYPE_ENTRANCE then
			-- 土地を検索
			local landEntity = LandSystem.GetLandHits(e.cell.xi, e.cell.yi);

			-- 商品がある商品棚を探す

			if landEntity ~= nil then
				local shelves = LandSystem.GetMapChips(landEntity, MapChipData.CATEGORY_STORE_SHELF);
				local availableShelves = {};

				for i = 0, shelves.Count - 1 do
					local s = shelves[i];
					if s.cell.children.Count > 0 then
						availableShelves[#availableShelves + 1] = s;
					end
				end

				if #availableShelves > 0 then
					local index = 1 + Random.rand(#availableShelves);
					shelf = availableShelves[index];
				end

				-- shelf = LandSystem.GetMapChips(landEntity, MapChipData.CATEGORY_STORE_SHELF)
				-- 	:Where(function(s) return s.cell.HasChildren() end)
				-- 	.ToArray()
				-- 	.RandomOrDefault();


				-- 棚がない
				if shelf == nil then
					ai.AddCommandFukidashi(BalloonFlags.SoldOut);
				end
			end
		end

		-- 棚がなければ、次の目標を探す
		if shelf == nil then
			ai.AddCommandWait(10);
			ai.AddCommandPhase(0);

			-- 待機アニメへ変更
			AnimationSet.ChangeAnimation(e, AnimationSet.BEHAVIOR_EQUIP_WAIT);
		-- 商品棚の前へ移動する
		else
			AISystem.MoveTo(e, shelf.cell.xi, shelf.cell.yi, shelf.direction.dir);
			ai.AddCommandChangeAnime(AnimationSet.BEHAVIOR_EQUIP_WAIT);
			ai.AddCommandPhase(4);

			ai.subject = shelf;
		end

		ai.phase = 3;

	-- 待機中/商品棚へ移動中
	elseif ai.phase == 3 then
	
	-- 商品棚へ着いた
	elseif ai.phase == 4 then
		local shelf = ai.subject;

		-- 棚から商品を取る
		local product = nil;
		for i = 0, shelf.cell.children.Count - 1 do
			local child = shelf.cell.children[i];
			if child.hasProduct then
				product = child;
				break;
			end
		end
		-- local product = StockSystem.PopStock(shelf.cell.children.FirstOrDefault(function(c) return c.hasProduct end), 1);

		-- 売切れてた
		if product == nil then
			ai.AddCommandFukidashi(BalloonFlags.SoldOut);
			ai.AddCommandWait(10);
			ai.AddCommandPhase(0);

			ai.phase = 5;

			return;
		end
			
		product.AddInvisible();
		product.position.parent = e;
		ai.subject2 = product;

		-- 持ち物の画像に、商品の画像を設定
		e.human.imgIds[HumanComponent.IMG_LIFT + 1] = product.image.texId;

		-- レジを探す
		local landEntity = LandSystem.GetLandHits(e.cell);
		local regi       = LandSystem.SearchMapChip(landEntity, MapChipData.TYPE_FURNITURE, MapChipData.CATEGORY_REGISTER); 
		ai.subject3    = regi;

		-- 商品を手に取ってレジへもっていく
		ai.AddCommandWait(5);
		ai.AddCommandChangeAnime(AnimationSet.BEHAVIOR_HAND);
		ai.AddCommandFukidashi(BalloonFlags.Shopping);
		ai.AddCommandWait(20);
		ai.AddCommandChangeAnime(AnimationSet.BEHAVIOR_LIFT_PRODUCT);
		AISystem.MoveToFrontOf(e, regi.cell.xi, regi.cell.yi, regi.direction.dir);
		ai.AddCommandPhase(6);

		ai.phase = 5;
	
	-- 商品をレジへ持っていく/売り切れ
	elseif ai.phase == 5 then
	
	-- レジへ到着
	elseif ai.phase == 6 then
		-- 買い物する
		local product = ai.subject2;
		local regi    = ai.subject3;

		-- 商品がない
		if product == nil or product.isDestroyed then
			Log.Info("客が持ってた商品が消えた");
			ai.phase = 0;
			return;
		end

		-- 商品をレジに置く
--					StockSystem.PushStock(product, regi);
		product.position.parent = nil;
		product.position.x = regi.position.x;
		product.position.y = regi.position.y;
		product.position.z = regi.position.z;
		product.RemoveInvisible();

		ai.AddCommandChangeAnime(AnimationSet.BEHAVIOR_HAND);
		ai.AddCommandFukidashi(BalloonFlags.BeforeBuy);
		ai.AddCommandWait(20);
		ai.AddCommandChangeAnime(AnimationSet.BEHAVIOR_EQUIP_WAIT);
		ai.AddCommandWait(20);
		ai.AddCommandChangeAnime(AnimationSet.BEHAVIOR_HAND);
		ai.AddCommandWait(20);
		ai.AddCommandFukidashi(BalloonFlags.AfterBuy);
		ai.AddCommandPhase(8);

		ai.phase = 7;

	-- 買い物中
	elseif ai.phase == 7 then
	
	-- 買い物終了、店を出る
	elseif ai.phase == 8 then
		local product = ai.subject2;

		if product ~= nil and not product.isDestroyed then
			-- 代金を払う
			-- TODO: 各種商品に対応する
			if product.product.dataType == DataTypes.Equip then
				local equip = product.product.equipData;
				local curreny = nil;

				for i, itemData in ipairs(Data.itemData) do
				-- for i = 1, #Data.itemData do
					-- local itemData = Data.itemData[i];
					if itemData.type == equip.currency then
						currency = itemData;
						break;
					end
				end
				-- Data.itemData.FirstOrDefault(function(d) return d.type == equip.currency end);

				if curreny ~= nil then
					if curreny.isUnfound then
						curreny.Find();
					end

					curreny.AddStock(equip.GetPrice(SellTarget.Customer));
				end
			elseif product.product.dataType == DataTypes.Item then
				local item = product.product.itemData;
				local curreny = nil;

				for i, itemData in ipairs(Data.itemData) do
				-- for i = 1, #Data.itemData do
					-- local data = Data.itemData[i];
					if itemData.type == equip.currency then
						currency = itemData;
						break;
					end
				end				
				-- local curreny = Data.itemData.FirstOrDefault(function(d) return d.type == item.currency end);

				if curreny ~= nil then
				
					if curreny.isUnfound then
						curreny.Find();
					end

					curreny.AddStock(item.GetPrice(SellTarget.Customer));
				end
			end
			-- apdat_.AddCoin(product.product.GetPrice(SellTarget.Customer));

			-- コインエフェクト
			World.world.CreateEffect(product.position.x, product.position.y, product.position.z, EffectComponent.TYPE_NORMAL, 0, AppData.RSRES_EFFECT, EffectIds.SebIds[EffectIds.GET_COIN], false, false);
			
			product.position.parent = e;
			product.AddInvisible();
		end

		-- 商品を運ぶアニメへ変更
--					AnimationSet.ChangeAnimation(e, AnimationSet.ANIMATION_BEHAVIOR_LIFT_PRODUCT);

		-- 土地を検索
		local landEntity = LandSystem.GetLandHits(e.cell);

		-- 玄関を探す
		local entrance = nil;
		if landEntity ~= nil then
			entrance = LandSystem.SearchMapChip(landEntity, MapChipData.TYPE_ENTRANCE, MapChipData.CATEGORY_ENTRANCE);
		end

		-- 玄関へ行く
		if entrance ~= nil then
			ai.AddCommandMoveRoute(entrance.cell.xi * World.world.map.stepX, entrance.cell.yi * World.world.map.stepZ);
			ai.AddCommandPhase(10);

			-- 待機アニメへ変更
			AnimationSet.ChangeAnimation(e, AnimationSet.BEHAVIOR_LIFT_PRODUCT);

			ai.phase = 9;
		else
			-- 再び行ける場所を探す
			ai.phase = 0;
		end

	-- 玄関へ移動中
	elseif ai.phase == 9 then
	
	-- 玄関へ着いた
	elseif ai.phase == 10 then
	
		-- 再び行ける場所を探す
		ai.phase = 0;
	end	
end

function updateReturnExit(e)
	local ai = e.ai;

	-- ai.subject2 = 商品

	-- 出口まで帰る準備
	if ai.phase == 0 then
		local vs = VisitorSystem.Instance;

		ai.AddCommandMoveRoute(vs.gateXi_ * World.world.map.stepX, vs.gateYi_ * World.world.map.stepZ);
		ai.AddCommandPhase(2);

		if ai.subject2 ~= nil and ai.subject2.hasProduct then
			-- 商品を運ぶアニメへ変更
			AnimationSet.ChangeAnimation(e, AnimationSet.BEHAVIOR_LIFT_PRODUCT);
		else
			-- 歩行アニメへ変更
			AnimationSet.ChangeAnimation(e, AnimationSet.BEHAVIOR_EQUIP_WALK);
		end

		ai.phase = 1;
	-- 出口へ移動中
	elseif ai.phase == 1 then

	-- 出口へ到着
	elseif ai.phase == 2 then
		-- フェードアウトする
		-- ModifyAnimationComponent.Type type = ModifyAnimationComponent.Type.Pop, float x = 0, float y = 0, float z = 0, float scaleX = 1, float scaleY = 1, int angle = 0, int frame = 0, int maxFrame = 0, int anchor = 0, bool destroy = false, bool loop = false, int alpha = 255
		e.AddModifyAnimation(ModifyAnimationComponent.Type.FadeOut, 0, 0, 0, 1, 1, 0, 0, 10, 0, true);

		-- 商品を持っていたら、破棄する
		if ai.subject2 ~= nil and ai.subject2.hasProduct then
			ai.subject2.Destroy();
		end

		ai.phase = 3;
	-- 消え去る
	elseif ai.phase == 3 then

	end
end

function onReload()

end
